#include <string.h>
#include <monocypher/monocypher.h>


#include "fk_crypt.h"


int check_keypair(const u_chr secret[], const u_chr pub[])
{
    int res;
    unsigned char secr_copy[secret_key_size];
    unsigned char gsecr[secret_key_size];
    unsigned char gpub[public_key_size];

    res = memcmp(pub, secret + (secret_key_size - public_key_size),
                 public_key_size);
    if(res != 0)
        return 0;

        /* we make a copy to avoid wiping the structure by the
           crypto_eddsa_key_pair function
         */
    memcpy(secr_copy, secret, secret_key_size);
    crypto_eddsa_key_pair(gsecr, gpub, secr_copy);
        /* this wipes the first half of secr_copy, which isn't a
           problem, as we don't need it any longer anyway; but its copy
           is made in the gsecr, which we need to wipe on our own
         */
    crypto_wipe(gsecr, sizeof(gsecr));

    res = memcmp(gpub, pub, public_key_size);
    if(res != 0)
        return 0;

    return 1;
}

const u_chr fedacert_magic[] = { 0xfe, 0xda, 0xce, 0x87 };

void compose_feda_cert(struct feda_cert_info *cert,
    const u_chr node_id[], int point_id, int signer_id, int timestamp,
    const u_chr key[])
{
    memcpy(cert->feda, fedacert_magic, sizeof(cert->feda));
    memcpy(cert->node_id, node_id, sizeof(cert->node_id));
    cert->point_id = point_id;
    cert->signer_id = signer_id;
    cert->timestamp[0] = (timestamp >> 24) & 0xff;
    cert->timestamp[1] = (timestamp >> 16) & 0xff;
    cert->timestamp[2] = (timestamp >>  8) & 0xff;
    cert->timestamp[3] = timestamp         & 0xff;
    memcpy(cert->key, key, sizeof(cert->key));
}

int get_cert_timestamp(const struct feda_cert_info *cert)
{
    return
        (((unsigned int)cert->timestamp[0]) << 24) |
        (((unsigned int)cert->timestamp[1]) << 16) |
        (((unsigned int)cert->timestamp[2]) <<  8) |
        (unsigned int)cert->timestamp[3]; 
}

const u_chr *get_cert_pubkey(const u_chr *cert)
{
    return ((struct feda_cert_info *)cert)->key;
}

int compare_feda_cert(const struct feda_cert_info *cert,
    const u_chr node_id[], int point_id, int signer_id, int timestamp,
    const u_chr key[])
{
    if(0 != memcmp(cert->feda, fedacert_magic, sizeof(cert->feda)))
        return 0;
    if(0 != memcmp(cert->node_id, node_id, sizeof(cert->node_id)))
        return 0;
    if(cert->point_id != point_id)
        return 0;
    if(cert->signer_id != (signer_id == -1 ? 0xff : signer_id))
        return 0;
    if(timestamp != -1 && timestamp != get_cert_timestamp(cert))
        return 0;
    if(0 != memcmp(cert->key, key, sizeof(cert->key)))
        return 0;
    return 1;
}


void compose_kex_info(struct feda_kex_info *kexinfo,
                      int timestamp, const u_chr kex_pub[])
{
    kexinfo->timestamp[0] = (timestamp >> 24) & 0xff;
    kexinfo->timestamp[1] = (timestamp >> 16) & 0xff;
    kexinfo->timestamp[2] = (timestamp >>  8) & 0xff;
    kexinfo->timestamp[3] = timestamp         & 0xff;
    memcpy(kexinfo->kex_public, kex_pub, sizeof(kexinfo->kex_public));
}
