#ifndef FK_CRYPT_H_SENTRY
#define FK_CRYPT_H_SENTRY

#include "cryptodf.h"

typedef unsigned char u_chr;     /* hope this won't conflict with anything */

struct feda_cert_info { /* all numbers are to be stored big-endian */
    u_chr feda[4];              /* magic number 0xFEDACE87 */
    u_chr node_id[node_id_size];
    u_chr point_id;
    u_chr signer_id;            /* 0xFF if signed by the node's master key,
                                   0x00 if signed by the PointZero's key   */
    u_chr timestamp[4];         /* minutes since epoch, a.k.a. unixtime/60 */
    u_chr key[public_key_size]; /* the key being certified */
};

struct feda_kex_info {
    u_chr timestamp[4];                  /* minutes since epoch */
    u_chr kex_public[kex_public_size];   /* the kex key being certified */
};

enum {
    feda_cert_size      = sizeof(struct feda_cert_info),
    feda_kex_info_size  = sizeof(struct feda_kex_info)
};

extern const u_chr fedacert_magic[];

void compose_feda_cert(struct feda_cert_info *cert,
    const u_chr node_id[], int point_id, int signer_id, int timestamp,
    const u_chr key[]);
int get_cert_timestamp(const struct feda_cert_info *cert);

const u_chr *get_cert_pubkey(const u_chr *cert);

    /* returns boolean; timestamp==-1 means no timestamp check */
int compare_feda_cert(const struct feda_cert_info *cert,
    const u_chr node_id[], int point_id, int signer_id, int timestamp,
    const u_chr key[]);

    /* returns boolean */
int check_keypair(const u_chr secret[], const u_chr pub[]);

void compose_kex_info(struct feda_kex_info *kexinfo,
                      int timestamp, const u_chr kex_pub[]);




#endif
